package com.mints.goodmoney.ui.activitys

import android.Manifest
import android.os.Bundle
import android.text.TextUtils
import android.view.View
import cn.jzvd.Jzvd
import cn.sharesdk.wechat.friends.Wechat
import com.dingmouren.videowallpaper.VideoWallpaper
import com.downloader.Error
import com.downloader.OnDownloadListener
import com.downloader.PRDownloader
import com.mints.goodmoney.R
import com.mints.goodmoney.ad.vedio.VedioAdingManager
import com.mints.goodmoney.call.detail.DetailActivity
import com.mints.goodmoney.call.util.Util
import com.mints.goodmoney.common.Constant
import com.mints.goodmoney.manager.UserManager
import com.mints.goodmoney.mvp.model.VedioAdingBean
import com.mints.goodmoney.mvp.presenters.KylVedioPresenter
import com.mints.goodmoney.mvp.views.KylVedioView
import com.mints.goodmoney.ui.activitys.base.BaseActivity
import com.mints.goodmoney.utils.CacheUtil
import com.mints.goodmoney.utils.LogUtil
import com.mints.goodmoney.utils.MD5
import com.tbruyelle.rxpermissions.RxPermissions
import kotlinx.android.synthetic.main.activity_kyl_vedio.*

/**
 * 描述：酷音乐视频
 * 作者：孟崔广
 * 时间：2021/1/4 15:09
 */
class KylVedioActivity : BaseActivity(), View.OnClickListener, KylVedioView {

    companion object {
        const val VEDIO_URL = "vedio_url"
    }

    private lateinit var vedioUrl: String

    private val kylVedioPresenter by lazy { KylVedioPresenter() }
    private lateinit var vedioAdingManager: VedioAdingManager
    private var loadVedioFailCount = 0

    private val videoWallpaper by lazy { VideoWallpaper() }
    private var downloadId: Int = -1//vedio下载id
    private var downloadVedioSucFlag = false// 下载视频成功

    override fun getContentViewLayoutID() = R.layout.activity_kyl_vedio

    override fun isApplyKitKatTranslucency() = false

    override fun getBundleExtras(extras: Bundle?) {
        super.getBundleExtras(extras)
        extras?.let {
            vedioUrl = it.getString(VEDIO_URL, "")
        }
    }

    override fun initViewsAndEvents() {
        if (TextUtils.isEmpty(vedioUrl)) {
            showToast("加载失败，请稍后重试.")
            finish()
            return
        }
        vedioAdingManager = VedioAdingManager.getInstance(this)
        initListener()
        loadVedio(vedioUrl)
    }

    private fun loadVedio(vedioUrl: String) {
        jzvdStd?.setUp(vedioUrl, "")
        jzvdStd?.startVideo()
//        jzvdStd.setVedioRestart(false)
//        jzvdStd.setJzVedioStatusListener(object : JZVideoPlayerStandardLoopVideo.JZVedioStatusListener {
//           override fun jzVedioPlaying() {
//            }
//        })
    }

    override fun onBackPressed() {
        if (Jzvd.backPress()) {
            return;
        }
        super.onBackPressed()
    }

    override fun onResume() {
        super.onResume()

        if (!isFinishing()) {
            Jzvd.goOnPlayOnResume()
        }
    }

    override fun onPause() {
        super.onPause()

        Jzvd.goOnPlayOnPause()
    }

    override fun onDestroy() {
        super.onDestroy()
        Jzvd.releaseAllVideos()
        jzvdStd?.removeAllViews()

        if (downloadId != -1) {
            PRDownloader.cancel(downloadId)
        }

        //删除缓存文件
        try {
            CacheUtil.clearAllCache(context)
        } catch (e: Exception) {
            e.printStackTrace()
        }

        kylVedioPresenter.detachView()
    }

    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.ivBack -> finish()
            R.id.ivSetting -> {
                RxPermissions(this)
                        .request(Manifest.permission.READ_EXTERNAL_STORAGE, Manifest.permission.WRITE_EXTERNAL_STORAGE)
                        .subscribe { granted: Boolean ->
                            if (granted) {
//                                downloadVedioToCacheFile()
                            } else {
                                showMissingPermissionDialog("存储")
                            }
                        }
//                awardVedio()
            }
        }
    }

    private fun downloadVedioToCacheFile() {
        showLoading("加载中...")
        var vedioPath = ""
        try {
            vedioPath = getExternalFilesDir(null)!!.absolutePath + "/video/"
        } catch (e: java.lang.Exception) {
            vedioPath = filesDir.absolutePath + "/video/"
        }
        val vedioName = MD5.GetMD5Code(System.currentTimeMillis().toString() + UserManager.getInstance().userID + vedioUrl) + ".mp4"
        downloadId = PRDownloader.download(vedioUrl, vedioPath, vedioName)
                .build()
                .start(object : OnDownloadListener {
                    override fun onDownloadComplete() {
                        hideLoading()
//                        downloadVedioSucFlag = true
                        val vedioName = vedioPath + vedioName
                        DetailActivity.startSelf(this@KylVedioActivity, vedioName)
                    }

                    override fun onError(error: Error) {
                        showToast("加载失败")
                        hideLoading()
                    }
                })
    }

    private fun initListener() {
        ivBack.setOnClickListener(this)
        ivSetting.setOnClickListener(this)
    }

    /**
     * 获取激励视频
     */
    private fun awardVedio() {
        if (vedioAdingManager.vedioFinishFlag) {
            showToast("今日视频已看完，请明天再来吧")
            return
        }
        loadVedioFailCount = 0
        showLoading("加载中...")

        val bean = VedioAdingBean()
        bean.carrierType = Constant.CARRIER_KUYINYUE_VEDIO
        loadVedio(vedioAdingManager, bean, true)
    }

    /**
     * 加载激励视频
     */
    private fun loadVedio(vedioAdingManager: VedioAdingManager, bean: VedioAdingBean, isFirstLoad: Boolean) {
        vedioAdingManager.setVedioAdingListener(object : VedioAdingManager.VedioAdingListener {
            override fun vedioAdingListenerError(adType: String) {
                showToast("加载超时，请休息一下")
                hideLoading()
            }

            override fun vedioAdingListenerFail(adType: String) {
                loadVedioFailCount++
                if (loadVedioFailCount >= 2) {
                    hideLoading()
                    showToast("加载失败，请稍后重试!")
                } else {
                    loadVedio(vedioAdingManager, bean, false)
                }
            }

            override fun vedioAdingListenerSuccess(adType: String) {
                hideLoading()
                vedioAdingSuccess(adType)
            }
        })
        if (isFirstLoad) {
            // 首页按权重加载
            vedioAdingManager.loadAding(this, bean)
        } else {
            // 失败按优先级加载
            vedioAdingManager.loadFailAding(this, bean)
        }
    }

    private fun vedioAdingSuccess(adType: String) {
        if (downloadVedioSucFlag) {
            try {
            } catch (e: Exception) {
                e.printStackTrace()
            }
        }
    }
}
